@@ -13,13 +13,13 @@ const login = require("../login");
 const customCSS = require("../customCSS");
 const Menus = require("../menus");
 const { SpellCheckProvider } = require("../spellCheckProvider");
-const { execFile } = require("child_process");
+const { execFile } = require("node:child_process");
 const TrayIconChooser = require("../browser/tools/trayIconChooser");
 require("../appConfiguration");
-const connMgr = require("../connectionManager");
+const ConnectionManager = require("../connectionManager");
 const BrowserWindowManager = require("../mainAppWindow/browserWindowManager");
-const os = require("os");
-const path = require("path");
+const os = require("node:os");
+const path = require("node:path");
 
 // Default configuration for the screen sharing thumbnail preview (avoid magic values)
 const DEFAULT_SCREEN_SHARING_THUMBNAIL_CONFIG = {
@@ -39,11 +39,45 @@ let window = null;
 let appConfig = null;
 let customBackgroundService = null;
 let streamSelector;
+let screenSharingService = null;
+let connectionManager = null;
 
 const isMac = os.platform() === "darwin";
 
+function findSelectedSource(sources, source) {
+  return sources.find((s) => s.id === source.id);
+}
+
+function setupScreenSharing(selectedSource) {
+  // Store the source ID in the screen sharing service
+  screenSharingService.setSelectedSource(selectedSource);
+
+  // Create preview window for screen sharing
+  createScreenSharePreviewWindow();
+}
+
+function handleScreenSourceSelection(source, callback) {
+  desktopCapturer
+    .getSources({ types: ["window", "screen"] })
+    .then((sources) => {
+      const selectedSource = findSelectedSource(sources, source);
+      if (selectedSource) {
+        setupScreenSharing(selectedSource);
+        callback({ video: selectedSource });
+      } else {
+        // Source not found - use setImmediate and try-catch to allow retry
+        setImmediate(() => {
+          try {
+            callback({});
+          } catch {
+            console.debug("[SCREEN_SHARE] Selected source not found");
+          }
+        });
+      }
+    });
+}
+
 function createScreenSharePreviewWindow() {
-  const path = require("path");
   const startTime = Date.now();
 
   // Get configuration - use the module-level config variable
@@ -52,11 +86,14 @@ function createScreenSharePreviewWindow() {
     config?.screenSharingThumbnail ??
     DEFAULT_SCREEN_SHARING_THUMBNAIL_CONFIG;
 
+  const previewWindow = screenSharingService.getPreviewWindow();
+  const activeSource = screenSharingService.getSelectedSource();
+
   console.debug("[SCREEN_SHARE_DIAG] Preview window creation requested", {
     enabled: thumbnailConfig.enabled,
     alwaysOnTop: thumbnailConfig.alwaysOnTop || false,
-    existingWindow: global.previewWindow && !global.previewWindow.isDestroyed(),
-    activeSource: global.selectedScreenShareSource,
+    existingWindow: previewWindow && !previewWindow.isDestroyed(),
+    activeSource: activeSource,
     timestamp: new Date().toISOString()
   });
 
@@ -66,13 +103,13 @@ function createScreenSharePreviewWindow() {
   }
 
   // Don't create duplicate windows - this is critical for preventing echo
-  if (global.previewWindow && !global.previewWindow.isDestroyed()) {
+  if (previewWindow && !previewWindow.isDestroyed()) {
     console.warn("[SCREEN_SHARE_DIAG] Preview window already exists, focusing existing", {
       riskLevel: "MEDIUM - multiple preview windows could cause audio issues",
       action: "focusing existing window instead of creating new",
-      windowId: global.previewWindow.id
+      windowId: previewWindow.id
     });
-    global.previewWindow.focus();
+    previewWindow.focus();
     return;
   }
 
@@ -82,7 +119,7 @@ function createScreenSharePreviewWindow() {
     partition: "persist:teams-for-linux-session"
   });
 
-  global.previewWindow = new BrowserWindow({
+  const newPreviewWindow = new BrowserWindow({
     width: 320,
     height: 180,
     minWidth: 200,
@@ -102,56 +139,62 @@ function createScreenSharePreviewWindow() {
     },
   });
 
-  const windowId = global.previewWindow.id;
+  // Store in service
+  screenSharingService.setPreviewWindow(newPreviewWindow);
+
+  const windowId = newPreviewWindow.id;
   console.debug("[SCREEN_SHARE_DIAG] Preview BrowserWindow created", {
     windowId: windowId,
     creationTimeMs: Date.now() - startTime,
     alwaysOnTop: thumbnailConfig.alwaysOnTop || false
   });
 
-  global.previewWindow.loadFile(
+  newPreviewWindow.loadFile(
     path.join(__dirname, "..", "screenSharing", "previewWindow.html")
   );
 
-  global.previewWindow.once("ready-to-show", () => {
+  newPreviewWindow.once("ready-to-show", () => {
     console.debug("[SCREEN_SHARE_DIAG] Preview window ready, showing now", {
       windowId: windowId,
       totalCreationTimeMs: Date.now() - startTime,
-      focused: global.previewWindow.isFocused(),
-      visible: global.previewWindow.isVisible()
+      focused: newPreviewWindow.isFocused(),
+      visible: newPreviewWindow.isVisible()
     });
-    global.previewWindow.show();
+    newPreviewWindow.show();
   });
 
   // Add focus/blur event handlers to detect when preview window gets focus
-  global.previewWindow.on("focus", () => {
+  newPreviewWindow.on("focus", () => {
     console.debug("[SCREEN_SHARE_DIAG] Preview window gained focus", {
       windowId: windowId,
       potentialIssue: "Focus on preview might interfere with main Teams window"
     });
   });
 
-  global.previewWindow.on("blur", () => {
+  newPreviewWindow.on("blur", () => {
     console.debug("[SCREEN_SHARE_DIAG] Preview window lost focus", {
       windowId: windowId
     });
   });
 
-  global.previewWindow.on("closed", () => {
+  newPreviewWindow.on("closed", () => {
+    const closedSource = screenSharingService.getSelectedSource();
     console.debug("[SCREEN_SHARE_DIAG] Preview window closed", {
       windowId: windowId,
-      hadActiveSource: !!global.selectedScreenShareSource,
-      closedSource: global.selectedScreenShareSource
+      hadActiveSource: !!closedSource,
+      closedSource: closedSource
     });
-    global.previewWindow = null;
-    global.selectedScreenShareSource = null;
+    // Clear both preview window and selected source when window closes
+    screenSharingService.setPreviewWindow(null);
+    screenSharingService.setSelectedSource(null);
   });
 }
 
-exports.onAppReady = async function onAppReady(configGroup, customBackground) {
+exports.onAppReady = async function onAppReady(configGroup, customBackground, sharingService) {
   appConfig = configGroup;
   config = configGroup.startupConfig;
   customBackgroundService = customBackground;
+  screenSharingService = sharingService;
 
   if (config.ssoInTuneEnabled) {
     intune = require("../intune");
@@ -202,38 +245,24 @@ exports.onAppReady = async function onAppReady(configGroup, customBackground) {
         if (source) {
           handleScreenSourceSelection(source, callback);
         } else {
-          callback({ video: null });
+          // User canceled - use setImmediate and try-catch to allow retry
+          setImmediate(() => {
+            try {
+              callback({});
+            } catch {
+              console.debug("[SCREEN_SHARE] User canceled screen selection");
+            }
+          });
         }
       });
     }
   );
 
-  function handleScreenSourceSelection(source, callback) {
-    desktopCapturer
-      .getSources({ types: ["window", "screen"] })
-      .then((sources) => {
-        const selectedSource = findSelectedSource(sources, source);
-        if (selectedSource) {
-          setupScreenSharing(selectedSource);
-        }
-        callback({ video: selectedSource });
-      });
-  }
-
-  function findSelectedSource(sources, source) {
-    return sources.find((s) => s.id === source.id);
-  }
-
-  function setupScreenSharing(selectedSource) {
-    // Store the source ID globally for access by screen sharing manager
-    global.selectedScreenShareSource = selectedSource;
-
-    // Create preview window for screen sharing
-    createScreenSharePreviewWindow();
-  }
+  // Initialize connection manager
+  connectionManager = new ConnectionManager();
 
   if (iconChooser) {
-    const m = new Menus(window, configGroup, iconChooser.getFile());
+    const m = new Menus(window, configGroup, iconChooser.getFile(), connectionManager);
     m.onSpellCheckerLanguageChanged = onSpellCheckerLanguageChanged;
   }
 
@@ -245,7 +274,7 @@ exports.onAppReady = async function onAppReady(configGroup, customBackground) {
   });
 
   const url = processArgs(process.argv);
-  connMgr.start(url, {
+  connectionManager.start(url, {
     window: window,
     config: config,
   });
@@ -263,6 +292,10 @@ exports.show = function () {
   window.show();
 };
 
+exports.getWindow = function () {
+  return window;
+};
+
 exports.onAppSecondInstance = function onAppSecondInstance(event, args) {
   console.debug("second-instance started");
   if (window) {
@@ -284,7 +317,7 @@ function applyAppConfiguration(config, window) {
   applySpellCheckerConfiguration(config.spellCheckerLanguages, window);
 
   if (
-    typeof config.clientCertPath !== "undefined" &&
+    config.clientCertPath !== undefined &&
     config.clientCertPath !== ""
   ) {
     app.importCertificate(
@@ -346,8 +379,7 @@ function onDidFinishLoad() {
 }
 
 function injectScreenSharingLogic() {
-  const fs = require("fs");
-  const path = require("path");
+  const fs = require("node:fs");
   const scriptPath = path.join(
     __dirname,
     "..",
@@ -524,8 +556,26 @@ function onPageTitleUpdated(_event, title) {
   window.webContents.send("page-title", title);
 }
 
+function onNavigationChanged() {
+  if (window?.webContents?.navigationHistory) {
+    const canGoBack = window.webContents.navigationHistory.canGoBack();
+    const canGoForward = window.webContents.navigationHistory.canGoForward();
+    window.webContents.send("navigation-state-changed", canGoBack, canGoForward);
+  }
+}
+
 function onWindowClosed() {
   console.debug("window closed");
+
+  // Close preview window before quitting to prevent race conditions
+  const previewWindow = screenSharingService?.getPreviewWindow();
+  if (previewWindow && !previewWindow.isDestroyed()) {
+    console.debug("[SCREEN_SHARE_DIAG] Closing preview window before app quit");
+    previewWindow.close();
+    screenSharingService.setPreviewWindow(null);
+    screenSharingService.setSelectedSource(null);
+  }
+
   window = null;
   app.quit();
 }
@@ -550,6 +600,10 @@ function addEventHandlers() {
   window.webContents.on("did-frame-finish-load", onDidFrameFinishLoad);
   window.on("closed", onWindowClosed);
   window.webContents.addListener("before-input-event", onBeforeInput);
+
+  // Navigation state change handlers
+  window.webContents.on("did-navigate", onNavigationChanged);
+  window.webContents.on("did-navigate-in-page", onNavigationChanged);
 }
 
 function getWebRequestFilterFromURL() {
@@ -595,14 +649,14 @@ function secureOpenLink(details) {
 }
 
 function openInBrowser(details) {
-  if (config.defaultURLHandler.trim() !== "") {
+  if (config.defaultURLHandler.trim() === "") {
+    shell.openExternal(details.url);
+  } else {
     execFile(
       config.defaultURLHandler.trim(),
       [details.url],
       openInBrowserErrorHandler
     );
-  } else {
-    shell.openExternal(details.url);
   }
 }
 
